/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "window.h"
#include "actions.h"
#include "client.h"
#include "menu.h"
#include "appmenu.h"


/*
 *----------------------------------------------------------------------
 * wCreateMenuFor--
 * 	Creates a menu for a window. Default items and app specified
 * items are included.
 * 
 * Returns:
 * 	A pointer to the menu that should be displayed when the client
 * is focused.
 * 
 * Side effects:
 * 	wwin->menu_owner is cleared if the menu belongs to another window
 * (like the leader of the group). 
 * 
 * Notes:
 * 	If a menu for an application subwindow is created before the
 * main window, the menu is created for the subwindow first, but
 * it can be replaced by the main windows menu, later.
 * 
 *---------------------------------------------------------------------- 
 */
WMenu *
wCreateMenuFor(WWindow *wwin)
{
    WMenu *menu;
    WScreen *scr=wwin->screen_ptr;
    char *title;

    /* transient windows share the menu with the main window */
    if (wwin->transient_for!=scr->root_win || wwin->group_id != None) {
	WWindow *main_win;
	
	main_win = wFindWWindow(wwin->transient_for);
	if (main_win) {
	    wwin->menu_owner = 0;
	    return main_win->menu;
	}
    }
    
    if (wwin->wm_class)
      title = wwin->wm_class;
    else if (wwin->wm_instance)
      title = wwin->wm_instance;
    else
      title = wwin->title;
    menu = wMenuCreate(scr, title);
    /*
    wMenuAddCascade(menu, "Windows", scr->window_menu);
     */
    wMenuAddItem(menu, "Hide", 13);
    wMenuAddItem(menu, "Quit", 14);
    wwin->menu_owner=1;
    menu->flags.app_menu=1;

    return menu;
}


