/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <stdio.h>

#include "WindowMaker.h"
#include "generic/wwmlib.h"
#include "core.h"
#include "box.h"


/* method table for box class */
static WMethodTable wBoxMethods = {
    NULL,			       /* mouse down */
      NULL,			       /* click */
      NULL			       /* double click */
};



WBox *
wBoxCreate(WCoreWindow *parent, int x, int y, int width, int height,
	   short border)
{
    WBox *box;
    
    box=wmalloc(sizeof(WBox));
    box->core = wCoreCreate(parent, x, y, width, height);
    box->core->descriptor.self = box;
    box->core->descriptor.methods = &wBoxMethods;
    
    box->border = border;
    box->texture=NULL;
    return box;
}


void
wBoxSetTexture(WBox *box, WTexture *texture)
{
    box->texture = texture;
}


void 
wBoxPaint(WBox *box, int repaint) 
{
#ifdef DEBUG
    if (box->texture==NULL) {
	puts("wBoxPaint() got NULL texture");
	return;
    }
#endif    
    wTexturePaint(box->core, box->texture, box->border, repaint);
}


void
wBoxDestroy(WBox *box)
{
    wCoreDestroy(box->core);
    wfree(box);
}



