/*
 *  WindowMaker miscelaneous function library
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "wwmlib.h"



/*
 *----------------------------------------------------------------------
 * find_file--
 * 	Finds a file in a : separated list of paths. ~ expansion is also 
 * done.
 * 
 * Returns:
 * 	The complete path for the file (in a newly allocated string) or
 * NULL if the file was not found.
 * 
 * Side effects:
 * 	A new string is allocated. This must be freed later.
 * 
 *---------------------------------------------------------------------- 
 */
#if 0
char *
find_file(char *paths, char *file)
{
    char *path;
    char cpath[MAXPATH], *fullpath;
    char *tmppath;
    int len;
    
    len = strlen(file);
    if (*file=='/' || !paths) {
	if (access(file, R_OK)<0)
	  return NULL;
	return strdup(file);
    }
    /* strtok is destructive */
    tmppath = strdup(paths);

    path=strtok(tmppath, ":");
    while (path) {
	strcpy(cpath,path);
	strcat(cpath,file);
	/* expand tilde */
	if (cpath[0]!='~') {
	    fullpath = strdup(cpath);
	} else {
	    char *home = getenv("HOME");
	    fullpath = xmalloc(strlen(home)+strlen(cpath));
	    strcpy(fullpath, home);
	    strcat(fullpath, &(cpath[1]));
	}
	if (access(fullpath, R_OK)==0) {
	    free(tmppath);
	    return fullpath;
	}
	free(fullpath);
	path=strtok(NULL, ":");
    }
    free(tmppath);
    return NULL;
}
#endif


char *
find_file(char *paths, char *file)
{
    char *path;
    char *tmp;
    int done;
    int len, flen;
    char *fullpath;

    if (*file=='/' || !paths) {
	if (access(file, R_OK)<0)
	  return NULL;
	return strdup(file);
    }
    flen = strlen(file);
    tmp = paths;
    done=0;
    while (!done) {
	len = strcspn(tmp, ":");
	if (len==0) done=1;
	path = xmalloc(len+flen+1);
	path = memcpy(path, tmp, len);
	path[len]=0;
	strcat(path, file);
	/* expand tilde */
	if (path[0]!='~') {
	    fullpath = path;
	} else {
	    char *home = getenv("HOME");

	    fullpath = xmalloc(strlen(home)+flen+len+1);
	    strcpy(fullpath, home);
	    strcat(fullpath, &(path[1]));
	    free(path);
	}
	if (access(fullpath, R_OK)==0) {
	    return fullpath;
	}
	free(fullpath);
	tmp=&(tmp[len+1]);
	if (*tmp==0) break;
    }
    return NULL;
}
