/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "window.h"
#include "funcs.h"
#include "menu.h"
#include "dialog.h"
#include "resource.h"
#ifdef SOUNDS
#include "sound.h"
#endif

/****** Global Variables ******/

/* general info */

Display	*dpy;
/* the root window */
char 	*ProgName;

WScreen wScreen;


WPreferences wPreferences;

/*
 * list of windows managed by us. WWindow->next points to the most recently
 * focused window, before itself */
WWindow *wWindowList=NULL;

/* modules */
int ModuleNo;

/* XContexts */
XContext wWinContext;

/* Atoms */
Atom _XA_WM_STATE;
Atom _XA_WM_CHANGE_STATE;
Atom _XA_WM_PROTOCOLS;
Atom _XA_WM_TAKE_FOCUS;
Atom _XA_WM_DELETE_WINDOW;
Atom _XA_WM_SAVE_YOURSELF;
Atom _XA_WM_NAME;
Atom _XA_WM_ICON_NAME;

Atom _XA_GNUSTEP_WM_ATTR;



/* cursors */
Cursor wCursor[WCUR_LAST];

/* last event timestamp for XSetInputFocus */
Time LastTimestamp;

/* paths */
char *MenuConfigPath=NULL;
char *PixmapPath=NULL;

/******** End Global Variables *****/

extern void EventLoop();
extern void StartUp();

/*
 *---------------------------------------------------------------------
 * wAbort--
 * 	Do a major cleanup and exit the program
 * 
 *---------------------------------------------------------------------- 
 */
void
wAbort()
{
    RestoreDesktop();
    printf("%s aborted.", ProgName);
    exit(1);
}


int
main(int argc, char **argv)
{
    ProgName = argv[0];

    MenuConfigPath = find_file(DEF_CONFIG_PATHS, DEF_MENUCONFIG);

    /* pixmap path can be freed later, if the user specified a new
     * path in configuration */
    PixmapPath = strdup(DEF_PIXMAP_PATHS);

    wResourceInitialize();
#ifdef SOUNDS    
    wSoundInitialize();
#endif
    StartUp("");
    wWindowInitialize();
    wMenuInitialize();
    wDialogInitialize();
    EventLoop();
    return -1;
}
