/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <unistd.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "core.h"
#include "window.h"
#include "config.h"
#include "client.h"
#include "icon.h"
#include "funcs.h"

/****** Global Variables ******/
extern Time LastTimestamp;

extern Cursor wCursor[WCUR_LAST];

extern WPreferences wPreferences;




static void
showPosition(WWindow *wwin, int x, int y)
{
    WScreen *scr=wwin->screen_ptr;
    GC gc = scr->line_gc;
    char num[16];
    int fw;

    fw = XTextWidth(scr->info_text_font, "8888x8888", 9);

    XClearWindow(dpy, scr->geometry_display);
    sprintf(num, "%4i x %4i", x, y);
    XDrawString(dpy, scr->geometry_display, gc, (80-fw)/2, 
		scr->info_text_font->ascent+4, num, strlen(num));
}


void
wMoveWindow(WWindow  *wwin)
{
    XEvent event;
    Window root = wwin->screen_ptr->root_win;
    GC gc = wwin->screen_ptr->frame_gc;
    KeyCode shiftl, shiftr;
    int w = wwin->frame->width;
    int h = wwin->frame->height;
    int x = wwin->frame_x;
    int y = wwin->frame_y;
    int dx, dy;
    Window junkw;
    int junk;

    XQueryPointer(dpy, root, &junkw, &junkw, &dx, &dy, 
		  &junk, &junk, (unsigned *)&junk);
    dx = -dx;
#ifdef DEBUG
    puts("Moving window");
#endif
    shiftl = XKeysymToKeycode(dpy, XK_Shift_L);
    shiftr = XKeysymToKeycode(dpy, XK_Shift_R);
    while(1) {
	XNextEvent(dpy, &event);
	switch (event.type) {
	 case KeyPress:
	    showPosition(wwin, x, y);
	    if ((event.xkey.keycode == shiftl) 
		|| (event.xkey.keycode == shiftr)) {
		XDrawRectangle(dpy, root, gc, x, y, w, h);		
		if (wPreferences.move_display==3)
		  wPreferences.move_display=0;
		else
		  wPreferences.move_display++;
		
		if (wPreferences.move_display == WDIS_NEW) {
		    XUnmapWindow(dpy, wwin->screen_ptr->geometry_display);
		} else {
		    WScreen *scr=wwin->screen_ptr;

		    if (wPreferences.move_display == WDIS_CENTER) {
			XMoveWindow(dpy, scr->geometry_display,
				    (scr->scr_width-80)/2,
				    (scr->scr_height-20)/2);
		    } else if (wPreferences.move_display == WDIS_TOPLEFT) {
			XMoveWindow(dpy, scr->geometry_display, 1, 1);
		    } else if (wPreferences.move_display==WDIS_FRAME_CENTER) {
			XMoveWindow(dpy, scr->geometry_display, x+w/2, y+h/2);
		    }
		    XMapRaised(dpy, scr->geometry_display);
		    showPosition(wwin, x, y);
		}
		XDrawRectangle(dpy, root, gc, x, y, w, h);
	    }
	    showPosition(wwin, x, y);
	    break;

	 case MotionNotify:
	    if (dx>=0) {
		showPosition(wwin, x, y);		
		XDrawRectangle(dpy, root, gc, x, y, w, h);
		x += event.xmotion.x_root - dx;
		y += event.xmotion.y_root - dy;
		dx = event.xmotion.x_root;
		dy = event.xmotion.y_root;
		if (wPreferences.move_display==WDIS_FRAME_CENTER) {
		    XMoveWindow(dpy, wwin->screen_ptr->geometry_display, 
				x+(w-80)/2, y+(h-20)/2);
		}
	    } else {
		XChangeActivePointerGrab(dpy, ButtonMotionMask
					 |ButtonReleaseMask, 
					 wCursor[WCUR_MOVE], CurrentTime);
		dx = -dx;
		XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, 
			      CurrentTime);
		XMapRaised(dpy, wwin->screen_ptr->geometry_display);
		showPosition(wwin, x, y);
		XGrabServer(dpy);
	    }
	    dx = event.xmotion.x_root;
	    dy = event.xmotion.y_root;
	    XDrawRectangle(dpy, root, gc, x, y, w, h);
	    break;
	    
	 case ButtonRelease:
	    if (dx>=0) {
		XDrawRectangle(dpy, root, gc, x, y, w, h);
		XSync(dpy, 0);
		wWindowConfigure(wwin, x, y, wwin->frame->width,
				 wwin->frame->height);
		XUngrabKeyboard(dpy, CurrentTime);
		XUngrabServer(dpy);
		/* get rid of the geometry window */
		XUnmapWindow(dpy, wwin->screen_ptr->geometry_display);
	    }

#ifdef DEBUG
	    puts("End move window");
#endif
	    return;

	 default:
	    DispatchEvent(&event);
	}
    }
}
 
#define LEFT	1
#define RIGHT	2
#define VERT	4




static int
getResizeDirection(WWindow  *wwin, int x, int y)
{
    int w = wwin->handle->core->width - 1;
    int cw = wwin->handle_corner_width;

    if ((x > cw + 5) && (x < w - (cw + 5)))
      return VERT;
    if (x < cw - 8)
      return LEFT;
    else {	
	if (x < cw + 5)
	  return (VERT|LEFT);
    }    
    if (x > w - (cw - 8))
      return RIGHT;
    return (RIGHT|VERT);
}


static void
showGeometry(WWindow *wwin, int x1, int y1, int x2, int y2, int direction)
{
    WScreen *scr=wwin->screen_ptr;
    Window root = scr->root_win;
    GC gc = scr->line_gc;
    int ty, by, my, x, y, mx, s;
    char num[16];
    int fw, fh=scr->info_text_font->ascent+scr->info_text_font->descent;

    ty = y1+wwin->top_width;
    by = y2-wwin->bottom_width;
    fw = XTextWidth(scr->info_text_font, "8888x8888", 9);
    if (wPreferences.size_display==WDIS_NEW) {
	/* vertical geometry */
	if (((direction & LEFT) && (x2<scr->scr_width-fw)) || (x1<fw)) {
	    x = x2;
	    s = -15;
	} else {
	    x = x1;
	    s = 15;
	}
	my = y1+(by-ty)/2+scr->info_text_font->ascent;
    
	XDrawLine(dpy, root, gc, x-s, ty, x-(s-5), ty);
	XDrawLine(dpy, root, gc, x-s, by, x-(s-5), by);
    
	XDrawLine(dpy, root, gc, x-(s-2), ty, x-(s-2), my);
	XDrawLine(dpy, root, gc, x-(s-2), my+fh, x-(s-2), by);
	
	sprintf(num, "%i", (by-ty-wwin->normal_hints->base_height)
		/wwin->normal_hints->height_inc);
	fw = XTextWidth(scr->info_text_font, num, strlen(num));
	XDrawString(dpy, root, gc, x-(s-2)-fw/2, 
		    my + scr->info_text_font->ascent + 1, num, strlen(num));
	
	/* horizontal geometry */
	if (y1<15) {
	    y = y2;
	    s = -15;
	} else {
	    y = y1;
	    s = 15;
	}
	mx = x1 + (x2-x1)/2;
	sprintf(num, "%i", (x2-x1)/wwin->normal_hints->width_inc);
	fw = XTextWidth(scr->info_text_font, num, strlen(num));
	XDrawLine(dpy, root, gc, x1, y-s, x1, y-(s-5));
	XDrawLine(dpy, root, gc, x2, y-s, x2, y-(s-5));
	
	XDrawLine(dpy, root, gc, x1, y-(s-2), mx-fw/2, y-(s-2));
	XDrawLine(dpy, root, gc, mx+fw/2, y-(s-2), x2, y-(s-2));
	
	XDrawString(dpy, root, gc, mx-fw/2, y-(s-2)+fh/2, num, strlen(num));

    } else {
	XClearWindow(dpy, scr->geometry_display);
	sprintf(num, "%4i x %4i", (by-ty-wwin->normal_hints->base_height)
		/wwin->normal_hints->height_inc, 
		(x2-x1)/wwin->normal_hints->width_inc);
	XDrawString(dpy, scr->geometry_display, gc, (80-fw)/2, 
		    scr->info_text_font->ascent+4, num, strlen(num));
    }
}


void
wResizeWindow(WWindow  *wwin)
{
    XEvent event;
    Window root = wwin->screen_ptr->root_win;
    GC gc = wwin->screen_ptr->frame_gc;
    int vert_border = wwin->top_width + wwin->bottom_width;    
    int fw = wwin->frame->width;
    int fh = wwin->frame->height;
    int fx = wwin->frame_x;
    int fy = wwin->frame_y;
    int dragx, dragy;
    int ox, oy;
    int dw, dh;
    int rw=fw, rh=fh;
    int rx1, ry1, rx2, ry2;
    int res=0;
    Window junkw;
    int junk;
    KeyCode shiftl, shiftr;
    
    XQueryPointer(dpy, root, &junkw, &junkw, &ox, &oy, &junk, &junk, 
		  (unsigned *)&junk);
    ox = -ox;
    
#ifdef DEBUG
    puts("Resizing window");
#endif
    
    rx1 = fx;
    rx2 = fx + fw - 1;
    ry1 = fy;
    ry2 = fy + fh - 1;
    ox = -1;
    shiftl = XKeysymToKeycode(dpy, XK_Shift_L);
    shiftr = XKeysymToKeycode(dpy, XK_Shift_R);
    while(1) {
	XNextEvent(dpy, &event);
	switch (event.type) {
	 case KeyPress:
	    showGeometry(wwin, fx, fy, fx+fw, fy+fh, res);
	    if ((event.xkey.keycode == shiftl) 
		|| (event.xkey.keycode == shiftr)) {
		XDrawRectangle(dpy, root, gc, fx, fy, fw, fh);		
		if (wPreferences.size_display==3)
		  wPreferences.size_display=0;
		else
		  wPreferences.size_display++;
		
		if (wPreferences.size_display == WDIS_NEW) {
		    XUnmapWindow(dpy, wwin->screen_ptr->geometry_display);
		} else {
		    WScreen *scr=wwin->screen_ptr;

		    if (wPreferences.size_display == WDIS_CENTER) {
			XMoveWindow(dpy, scr->geometry_display,
				    (scr->scr_width-80)/2, 
				    (scr->scr_height-20)/2);
		    } else if (wPreferences.size_display == WDIS_TOPLEFT) {
			XMoveWindow(dpy, scr->geometry_display, 1, 1);
		    } else if (wPreferences.size_display==WDIS_FRAME_CENTER) {
			XMoveWindow(dpy, scr->geometry_display, fx+(fw-80)/2, 
				    fy+(fh-20)/2);
		    }
		    XMapRaised(dpy, scr->geometry_display);
		    showGeometry(wwin, fx, fy, fx+fw, fy+fh, res);
		}
		XDrawRectangle(dpy, root, gc, fx, fy, fw, fh);
	    }
	    showGeometry(wwin, fx, fy, fx+fw, fy+fh, res);
	    break;

	 case MotionNotify:
	    dragx = event.xmotion.x_root;
	    dragy = event.xmotion.y_root;
	    if (ox>=0) {
		showGeometry(wwin, fx, fy, fx+fw, fy+fh, res);
		XDrawRectangle(dpy, root, gc, fx, fy, fw, fh);
		dw = dragx - ox;
		dh = dragy - oy;
		if (res & LEFT) {
		    dw = -dw;
		} else if (res & RIGHT) {
		    /* */
		} else { /* vertical only */
		    dw = 0;
		}

		if (!(res & VERT)) {
		    dh = 0;
		}
		rw += dw;
		rh += dh;
		
		fw = rw;
		fh = rh - vert_border;
		wWindowConstrainSize(wwin, &fw, &fh);		
		fh += vert_border;

		if (res & LEFT) {
		    fx = rx2 - fw;
		} else if (res & RIGHT) {
		    fx = rx1;
		}
		fy = ry1;
		if (wPreferences.size_display==WDIS_FRAME_CENTER) {
		    XMoveWindow(dpy, wwin->screen_ptr->geometry_display, 
				fx+(fw-80)/2, fy+(fh-20)/2);
		}
	    } else {
		if (wPreferences.size_display!=WDIS_NEW) {
		    XMapRaised(dpy, wwin->screen_ptr->geometry_display);
		}
		XTranslateCoordinates(dpy, root, wwin->handle->core->window,
				      event.xmotion.x_root,
				      event.xmotion.y_root, &ox, &oy, &junkw);
		res = getResizeDirection(wwin, ox, oy);
		XChangeActivePointerGrab(dpy, ButtonMotionMask
					 |ButtonReleaseMask,
					 wCursor[WCUR_MOVE], CurrentTime);
		XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, 
			      CurrentTime);
#ifndef NOGRAB		
		XGrabServer(dpy);
#endif		
	    }
	    ox = dragx;
	    oy = dragy;
	    showGeometry(wwin, fx, fy, fx+fw, fy+fh, res);
	    XDrawRectangle(dpy, root, gc, fx, fy, fw, fh);
	    break;
	    
	 case ButtonRelease:
	    if (ox>=0) {
		showGeometry(wwin, fx, fy, fx+fw, fy+fh, res);		
		XDrawRectangle(dpy, root, gc, fx, fy, fw, fh);
		wWindowConfigure(wwin, fx, fy, fw, fh);
		XUngrabKeyboard(dpy, CurrentTime);
		XUnmapWindow(dpy, wwin->screen_ptr->geometry_display);
#ifndef NOGRAB
		XUngrabServer(dpy);
#endif		
	    }
#ifdef DEBUG
	    puts("End resize window");
#endif
	    return;
	 default:
	    DispatchEvent(&event);
	}
    }
}

#undef LEFT
#undef RIGHT
#undef VERT

