/*
 * raster.h- Raster graphics function (dithering, quantization etc.) library
 * 
 * RasterLib
 * 
 * Copyright (c) 1997 Alfredo K. Kojima
 */

#ifndef RASTERLIB_H_
#define RASTERLIB_H_

#include <X11/Xlib.h>

/*
 * describes a screen in terms of depth, visual, number of colors
 * we can use, if we should do dithering, and what colors to use for
 * dithering.
 */
typedef struct RScreenDescriptor {
    int screen_no;		       /* number of the screen */
    Window parent;		       /* window to use as root (visual must
					 match the desired one) */
    Visual *visual;
    int depth;			       /* depth of the screen */
    struct {
	int dither:1;		       /* use dithering? */
	int direct_color:1;	       /* is this visual direct color? */
    } flags;
    /* only used for non-DirectColor visuals */ 
    Colormap cmap;		       /* X colormap */   
    int r_colors;		       /* colors are allocated like */
    int g_colors;		       /* color_no=r_colors*g_colors*b_colors*/
    int b_colors;		       
    int color_no;		       /* total number of colors we can use */
    XColor *colors;		       /* array of colors to be used */
} RScreenDescriptor;


/*
 * internal image representation
 */
typedef struct RImage {
    int width, height;		       /* size of the image */
    char *data;			       /* image data */
    char *alpha;		       /* alpha channel mask */
    int depth;			       /* depth of the image */
} RImage;

#endif
