/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef WMSCREEN_H_
#define WMSCREEN_H_


#include "texture.h"

typedef struct WScreen {
    int	screen;			       /* screen number */
    Window root_win;		       /* root window of screen */
    Visual *visual;		       /* the default visual to use */
    int  depth;			       /* depth of the default visual */

    Window w_win;		       /* window to use as drawable
					* for new GCs, pixmaps etc. */
    Visual *w_visual;
    int  w_depth;
    
    unsigned long black_pixel;
    unsigned long white_pixel;
    XFontStruct *title_font;	       /* default font for the titlebars */
    XFontStruct *menu_title_font;      /* font for menu titlebars */
    XFontStruct *menu_entry_font;      /* font for menu items */
    XFontStruct *icon_title_font;      /* for icon titles */
    XFontStruct *panel_title_font;     /* dialog box title */
    XFontStruct *panel_text_font;      /* dialog box text */
    XFontStruct *info_text_font;       /* text on things like geometry 
					* hint boxes */

    Colormap colormap;
    
    int scr_width;		       /* size of the screen */
    int scr_height;
    
    WTexture *focused_texture;	       /* default focused win texture */
    WTexture *unfocused_texture;       /* default unfocused win texture */
    WTexture *pushed_button_texture;   /* pushed buttons */
    WTexture *icon_texture;	       /* texture for icons */

    GC focused_text_gc;		       /* def. focused text GC*/
    GC unfocused_text_gc;	       /* def. unfocused text GC */
    GC icon_title_gc;		       /* icon title background */
    
    GC menu_title_gc;		       /* menu titles */
    GC menu_entry_gc;		       /* menu entries */
    GC select_menu_gc;		       /* selected menu entries */
    GC disabled_menu_entry_gc;	       /* disabled menu entries */

    GC frame_gc;		       /* gc for resize/move frame (root) */
    GC line_gc;			       /* gc for drawing XORed lines (root) */
    GC copy_gc;			       /* gc for XCopyArea() */
    struct WPixmap *b_pixmaps[PRED_BPIXMAPS]; /* internal pixmaps for buttons*/

    struct WPixmap *button_arrow;       /* arrow pixmap for panel buttons */
    int app_menu_x, app_menu_y;	       /* position for application menus */

    struct WMenu *root_menu;	       /* root window menu */
    
    Window geometry_display;	       /* displays the geometry during 
					* window resize, move etc. */
    struct WPixmap *default_icon;      /* default pixmap for icons */
} WScreen;


WScreen *wScreenInit(int screen_number);


#endif
