/*
 * yamaha.c- OPL3 sound support
 * 
 * WindowMaker
 *
 * Copyright (c) 1997 Alfredo K. Kojima
 * 
 * heavily based on fmtest, which has the following copyright:
 */
/* FMTEST by Rob Hooft (hooft@chem.ruu.nl) */
/*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 2.
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 */



#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <sys/soundcard.h>

#include "../generic/wwmlib.h"


#define MAX_SOUNDS 1

/* volume to play sounds */
int SndVolume=64;

/* instruments */
static struct sbi_instrument Instruments[MAX_SOUNDS];

/* array of filenames to use for instruments */
static char *SoundFiles[MAX_SOUNDS]={
    "sound/snrsust.sbi"
};


static int FMdev;
static unsigned char SBbuf[404];
static int sbptr = 0;

static void 
sbflush()
{
	if (!sbptr) return;

	if (write(FMdev, SBbuf, sbptr) == -1)
	{
	    perror("write sb");
	    return;
	}

	sbptr=0;
}

static void 
sbwrite(char msg[4])
{
    if (sbptr>400) sbflush();

    memcpy(&SBbuf[sbptr], msg, 4);
    sbptr +=4;
}


static void 
noteon(int pitch,int vol)
{
    char buf[4];

    buf[0] = SEQ_FMNOTEON;
    buf[1] = 0;
    buf[2] = pitch;
    buf[3] = vol;
    sbwrite(buf);	
}

static void 
noteoff(int pitch,int vol)
{
    char buf[4];

    buf[0] = SEQ_FMNOTEOFF;
    buf[1] = 0;
    buf[2] = pitch;
    buf[3] = vol;
    sbwrite(buf);	
}


static void 
wait(int delay)
{
    int jiffies;
    jiffies = (delay << 8) | SEQ_WAIT;
    sbwrite((char*)&jiffies);
}



void
wSoundInitialize()
{
    int i,j, l;
    int instrf;
    char buf[128];

    for (i=0; i<MAX_SOUNDS; i++) {
	if (SoundFiles[i]==NULL) { 
	    Instruments[i].device = -1;
	    continue;
	}
	
	if ((instrf=open(SoundFiles[i], O_RDONLY, 0))==-1) {
	    wSysError("%s", SoundFiles[i]);
	} else if ((l=read(instrf, buf, 100))==-1) {
	    wSysError("%s", SoundFiles[i]);
	} else if (buf[0]!='S' || buf[1]!='B' || buf[2]!='I') {
	    wWarning("%s is not a SBI file", SoundFiles[i]);
	} else if (l<51) {
	    wWarning("%s: Short file\n",SoundFiles[i]);
	} else {
	    Instruments[i].channel = 1;
	    for (j=0;j<16;j++) {
		Instruments[i].operators[j]=buf[j+0x24];
	    }
	    close(instrf);
	}
	/*
	free(SoundFiles[i]);
	  */
    }
}


void
wSoundPlay(int sound)
{
    char buf[4];
    int octave = 2, time = 50;
    
    if (Instruments[sound].device<0) return;
    
    if ((FMdev=open("/dev/sequencer", O_WRONLY|O_NONBLOCK))==-1) {
#ifdef DEBUG
	wSysError("could not open /dev/sequencer");
#endif	
	return;
    }

    if (ioctl(FMdev, SNDCTL_FM_LOAD_INSTR, &Instruments[sound])==-1) {
#ifdef DEBUG	
	wSysError("error trying to play FM sound");
#endif	
	return;
    }
    buf[0] = SEQ_FMPGMCHANGE;
    buf[1] = 0;
    buf[2] = 0;
    sbwrite(buf);
    noteon(12*octave,SndVolume); 
    wait(1*time);
    noteoff(12*octave,SndVolume);
    sbflush();
    close(FMdev);
}
