/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "wconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "window.h"


extern WPreferences wPreferences;

/*
 *----------------------------------------------------------------------
 * InstallColormap--
 * 	Does colormap installation for the screen. If wwin is NULL,
 * the root colormap is installed.
 * 
 * Notes:
 * 	Called from the EnterNotify event handler or from wSetFocusTo(),
 * depending on the colormap installation mode. 
 * 	Not completed yet.
 *---------------------------------------------------------------------- 
 */
void
InstallColormap(WScreen *screen, WWindow *wwin)
{
    XWindowAttributes attr;

    if (!wwin) {
	XInstallColormap(dpy, screen->colormap);
	screen->current_colormap=screen->colormap;
	return;
    }

    if (!XGetWindowAttributes(dpy, wwin->client_win, &attr))
      return;
    if (attr.colormap!=screen->current_colormap && attr.colormap!=None) {
	screen->current_colormap = attr.colormap;
	XInstallColormap(dpy, attr.colormap);
    } else if (attr.colormap==None 
	       && screen->current_colormap!=screen->colormap) {
	XInstallColormap(dpy, screen->colormap);
	screen->current_colormap=screen->colormap;
    }
}



void
ChangeColormap(WWindow *wwin, XEvent *event)
{
    XColormapEvent *xcme = &event->xcolormap;
    
    if (xcme->new && (wPreferences.focus_mode!=WKF_CLICK ||
		      wwin==wwin->screen_ptr->focused_window)) {
	InstallColormap(wwin->screen_ptr, wwin);
    } else {
	/* some obsolete client changed it's colormap,
	 * so install the real colormap back */
	XInstallColormap(dpy, wwin->screen_ptr->current_colormap);
	XSync(dpy, 0);
    }
}
