/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "wconfig.h"

#include <X11/Xlib.h>
#include <stdlib.h>
#include <stdio.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "wcore.h"
#include "icon.h"
#include "actions.h"
#include "funcs.h"


extern WPreferences wPreferences;

/*
 * interactive window placement is in moveres.c
 */

extern void
InteractivePlaceWindow(WWindow *wwin, int *x_ret, int *y_ret);


void
PlaceIcon(WScreen *scr, int *x_ret, int *y_ret)
{
    int x1, y1, x2, y2;
    WWindow *wwin;

    x1 = 0;
    y2 = scr->scr_height;
    y1 = y2-ICON_HEIGHT*2;    
    x2 = ICON_WIDTH;

    /* Maybe this is NP, isn't it? I hope nobody opens 1000+ windows... */
    while (1) {
	wwin = scr->focused_window;
	while (wwin) {
	    if (x2>=scr->scr_width) {
		x1 = 0;
		x2 = ICON_WIDTH*2;
		y1 -= ICON_HEIGHT;
		y2 -= ICON_HEIGHT;
		if (y2<ICON_HEIGHT) {
		    /* what's this guy doing!? */
		    *x_ret = 0;
		    *y_ret = 0;
		    return;
		}
		break;
	    }
	    if ((wwin->flags.miniaturized || wwin->flags.hidden)
		&& wwin->icon_y>y1 && wwin->icon_y<y2) {
		if (wwin->icon_x<x2 && wwin->icon_x>=x1) { 
		    x2 = wwin->icon_x+ICON_WIDTH*2;
		    x1 = wwin->icon_x+ICON_WIDTH;
		    /* this place can't be used */
		    wwin = scr->focused_window;
		    break;
		}
	    }
	    wwin = wwin->prev;
	}
	if (!wwin) {
	    /* found spot */
	    break;
	}
    }
    *x_ret = x1;
    *y_ret = y2-ICON_HEIGHT;
}



void
PlaceWindow(WWindow *wwin, int *x_ret, int *y_ret)
{
    WScreen *scr = wwin->screen_ptr;
    int h = scr->title_font->ascent+scr->title_font->descent+8;

    switch (wPreferences.window_placement) {
     case WPM_MANUAL:
	InteractivePlaceWindow(wwin, x_ret, y_ret);
	break;

     case WPM_CASCADE:
	scr->cascade_index++;
	*x_ret = h*scr->cascade_index;
	*y_ret = h*scr->cascade_index;
	if ((*x_ret > scr->scr_width/2)||(*y_ret > scr->scr_height/2))
	  scr->cascade_index = 0;
	break;
#ifdef DEBUG	
     default:
	puts("Invalid window placement!!!");
	*x_ret = 0;
	*y_ret = 0;
#endif
    }
}

