/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef WMRASTER_H_
#define WMRASTER_H_

#include <X11/Xlib.h>
#include "screen.h"
#include "pixmap.h"

/*
 * describes a screen in terms of depth, visual, number of colors
 * we can use, if we should do dithering, and what colors to use for
 * dithering.
 */
typedef struct WScreenDescriptor {
    int screen_no;		       /* number of the screen */
    Window parent;		       /* window to use as root (visual must
					 match the desired one) */
    Visual *visual;
    int depth;			       /* depth of the screen */
    GC copy_gc;

    struct {
	int dither:1;		       /* use dithering? */ 
	int true_color:1;	       /* if visual is direct/true color
					* or PseudoColor with a TrueColor-
					* like colormap */
	int mono:1;		       /* use monochrome dithering? */
    } flags;
    
    unsigned long black;
    unsigned long white;
    /* only used for non-DirectColor visuals */ 
    Colormap cmap;		       /* X colormap */   
    int r_colors;		       /* colors are allocated like */
    int g_colors;		       /* color_no=r_colors*g_colors*b_colors*/
    int b_colors;		       
    int ncolors;		       /* total number of colors we can use */
    
    int r_shift;
    int g_shift;
    int b_shift;
    
    XColor *colors;		       /* internal colormap */
} WScreenDescriptor;


/*
 * internal 24bit image representation
 */
typedef struct WImage {
    int width, height;		       /* size of the image */
    unsigned char *data;	       /* image data  */
    unsigned char *alpha;	       /* alpha channel mask */
} WImage;



WScreenDescriptor *wInitRaster(WScreen *scr, int r_weight, 
			       int g_weight, int b_weight);

void AllocColor(WScreen *screen, XColor *color);

#ifdef DITHERING
int DitherImage(WScreenDescriptor *ws, WImage *image, Pixmap *pixmap);

WImage *LoadImageFile(WScreen *scr, char *path);
#else
#define AllocColor(screen, color) XAllocColor(dpy, (screen)->colormap, color);
#endif /* DITHERING */

Pixmap LoadXPMIcon(WScreenDescriptor *ws, WImage *back, int titleh, 
		    char *file);


#endif
