/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef WMWINDOW_H_
#define WMWINDOW_H_

#include "wconfig.h"
#include "GNUstep.h"
#include "texture.h"
#include "button.h"
#include "menu.h"
#include "application.h"

/* not defined in my X11 headers */
#ifndef UrgencyHint
#define UrgencyHint (1L << 8)
#endif


#define BORDER_TOP	1
#define BORDER_BOTTOM	2
#define BORDER_LEFT	4
#define BORDER_RIGHT	8
#define BORDER_ALL	(1|2|4|8)


#define CLIENT_EVENTS (StructureNotifyMask | PropertyChangeMask\
	| EnterWindowMask | LeaveWindowMask | ColormapChangeMask \
	| FocusChangeMask)

typedef enum {
    WFM_PASSIVE, WFM_NO_INPUT, WFM_LOCALLY_ACTIVE, WFM_GLOBALLY_ACTIVE
} FocusMode;


/*
 * window attribute flags.
 * 
 * If you change, add, remove or reorder something in this structure,
 * make the appropriate changes in the bit masks in resource.c
 */
typedef struct {
    /* OpenStep */
    unsigned int titlebar:1;	       /* draw titlebar? */
    unsigned int resizable:1;	       /* draw the bottom handle? */
    unsigned int closable:1;	       /* draw a close button? */
    unsigned int miniaturizable:1;     /* draw an iconify button? */
    
    unsigned int broken_close:1;       /* is the close button broken? */
    /* ours */
    unsigned int shadeable:1;
    unsigned int focusable:1;
    unsigned int omnipotent:1;
    unsigned int skip_window_list:1;
    unsigned int on_top:1;	       /* keep window on top of all others 
					* of the same window level */
    unsigned int ignore_keys:1;	       /* do not intercept wm kbd binds 
					* while window is focused */
    unsigned int ignore_mouse:1;       /* do not intercept mouse events
					* on client area while window
					* is focused */
    unsigned int reserved:20;
} WWindowAttributes;



/*
 * Window manager protocols that both the client as we understand.
 */
typedef struct {
    unsigned int TAKE_FOCUS:1;
    unsigned int SAVE_YOURSELF:1;
    unsigned int DELETE_WINDOW:1;
} WProtocols;


/*
 * Stores client window information. Each client window has it's
 * structure created when it's being first mapped.
 */
typedef struct WWindow {
    struct WWindow *above;	       /* window stacking list. window
					* above of this one */
    struct WWindow *under;	       /* window under this one */

    struct WWindow *prev;	       /* circular window focus list */
    struct WWindow *next;

    WScreen *screen_ptr;	       /* pointer to the screen */
    WWindowAttributes window_flags;    /* window attribute flags */
    short window_level;		       /* window level (stacking order) */
    char workspace;		       /* workspace that the window occupies */

    WCoreWindow *frame;		       /* the frame window */
    int frame_x, frame_y;	       /* position of the frame in root*/
    short top_width, bottom_width;     /* width of the borders */

    struct {
	int x, y;
	int width, height;	       /* original geometry of the window */
    } old_geometry;		       /* (before things like maximize) */
    
    WCoreWindow *titlebar;	       /* the titlebar */
    WCoreWindow *handle;	       /* bottom resize handle */
    int handle_corner_width;

    WButton *l_buttons[MAX_LBUTTONS];  /* array of button at left */
    WButton *r_buttons[MAX_RBUTTONS];  /* array of button at right */
    short l_button_no;		       /* number of buttons defined */
    short r_button_no;

    WTexture *focused_texture;	       /* texture for focused win */
    WTexture *unfocused_texture;       /* texture for unfoc.win */
    WTexture *handle_texture;	       /* texture for handles */

    GC focused_text_gc;		       /* GC for drawing text for focused win*/
    GC unfocused_text_gc;	       /* GC for drawing text for unfoc win*/
    XFontStruct *title_font;	       /* font for the titlebar */
    
    /* client window info */
    Window client_win;		       /* the window we're managing */
    WObjDescriptor client_descriptor; /* dummy descriptor for client */
    int old_border_width;	       /* original border width of client_win */
    struct {
	int x, y;		       /* position of *client* relative 
					* to root */
	int width, height;	       /* size of the client window */
    } client;

    XSizeHints *normal_hints;	       /* WM_NORMAL_HINTS */
    XWMHints *wm_hints;		       /* WM_HINTS (optional) */
    char *wm_instance;		       /* instance of WM_CLASS */
    char *wm_class;    		       /* class of WM_CLASS */
    GNUstepAttributes *wm_gnustep_attr;/* GNUstep window attributes */
    
    char *title;		       /* window name (title) */
    int title_width;		       /* width of the title */
 
    int state;			       /* state as in ICCCM */
    
    WApplication *belongs_to;	       /* NULL or the application instance */
					/* it belongs to */
    Window transient_for;	       /* WM_TRANSIENT_FOR */
    Window group_id;		       /* the leader window of the group */
    Window client_leader;	       /* WM_CLIENT_LEADER */
      
    int cmap_window_no;
    Window *cmap_windows;
    
    /* protocols */
    WProtocols protocols;	       /* accepted WM_PROTOCOLS */
    
    FocusMode focus_mode;	       /* focus mode used  */    

    struct {
	/* state flags */
	unsigned int mapped:1;	
	unsigned int focused:1;
	unsigned int miniaturized:1;
	unsigned int hidden:1;
	unsigned int shaded:1;
	unsigned int maximized:1;
	unsigned int visible:1;
	
	/* window type flags */
	unsigned int urgent:1;	       /* if wm_hints says this is urgent */
	unsigned int group_leader:1;
#ifdef SHAPE
	unsigned int shaped:1;
#endif
	
	/* info flags */
	unsigned int no_buttons:1;
	unsigned int needs_full_repaint:1;/* does a full repaint of the 
					   * window next time it's painted */
	unsigned int icon_moved:1;     /* icon for this window was moved
					* by the user */
	unsigned int ignore_next_unmap:1;
	unsigned int selected:1;	
    } flags;		/* state of the window */

    char *icon_file;		       /* icon file to use (NULL == use 
							    global default)*/
    char *icon_mask_file;	       /* icon mask for XBM files */
    struct WIcon *icon;		       /* icon info for the window */
    int icon_x, icon_y;		       /* position of the icon */
} WWindow;



void wWindowDestroy(WWindow *wwin);
WWindow *wWindowCreate();

#ifdef SHAPE
void wWindowSetShape(WWindow *wwin);
#endif

WWindow *wManageWindow(WScreen *scr, Window window);

void wUnmanageWindow(WWindow *wwin, int restore);

void wWindowPaint(WWindow *wwin, int side, int repaint);
void wWindowFocus(WWindow *wwin);
void wWindowUnfocus(WWindow *wwin);
void wWindowConstrainSize(WWindow *wwin, int *nwidth, int *nheight);
void wWindowConfigure(WWindow *wwin, int req_x, int req_y, 
		      int req_width, int req_height);
WWindow *wWindowFor(Window window);

void wWindowInitialize();

void wWindowConfigureBorders(WWindow *wwin);

#endif
