/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef WMSCREEN_H_
#define WMSCREEN_H_

#include "wconfig.h"
#include "WindowMaker.h"
#include "texture.h"


typedef struct WFont {
#ifndef I18N
    XFontStruct *font;
#else
    XFontSet font;
#endif
    short height;
    short y;
} WFont;


typedef struct WScreen {
    int	screen;			       /* screen number */
    Window root_win;		       /* root window of screen */
    Visual *visual;		       /* the default visual to use */
    int  depth;			       /* depth of the default visual */
    Colormap colormap;		       /* root colormap */
    
    Window w_win;		       /* window to use as drawable
					* for new GCs, pixmaps etc. */
    Visual *w_visual;
    int  w_depth;
    Colormap w_colormap;	       /* our colormap */
    
    Window no_focus_win;	       /* window to get focus when nobody
					* else can do it */

    Colormap current_colormap;	       /* the colormap installed now */
    struct WWindow *focused_window;    /* window that has the focus */
    struct WCoreWindow *stacking_list[MAX_WINDOW_LEVELS];
    				       /* array of lists of windows
					* in stacking order.
					* The array order is in window level
					* order and each list on the array
					* is ordered from the topmost to
					* the lowest window
					*/
    int window_level_count[MAX_WINDOW_LEVELS];
    int window_count;		       /* number of windows in window_list */
    
    char workspace;		       /* current workspace number */
    char last_workspace;	       /* last workspace number */
    
    struct WScreenDescriptor *rscreen; /* descriptor for raster graphics lib */
    
    unsigned long black_pixel;
    unsigned long white_pixel;
    Pixmap stipple_bitmap;

    WFont title_font;		       /* default font for the titlebars */
    WFont menu_title_font;	       /* font for menu titlebars */
    WFont menu_entry_font;	       /* font for menu items */
    WFont icon_title_font;	       /* for icon titles */
    WFont panel_title_font;	       /* dialog box title */
    WFont panel_text_font;	       /* dialog box text */
    WFont info_text_font;	       /* text on things like geometry
					* hint boxes */
    
    int scr_width;		       /* size of the screen */
    int scr_height;
    
    WTexture *menu_title_texture;      /* menu titlebar texture */
    WTexture *menu_item_texture;       /* menu item texture */
    WTexture *focused_texture;	       /* default focused win texture */
    WTexture *unfocused_texture;       /* default unfocused win texture */
    WTexture *handle_texture;	       /* window handle texture */
    WTexture *normal_button_texture;   /* unpushed buttons */
    WTexture *pushed_button_texture;   /* pushed buttons */
    WTexSolid *icon_title_texture;     /* icon titles */
    
    GC focused_text_gc;		       /* def. focused text GC*/
    GC unfocused_text_gc;	       /* def. unfocused text GC */
    GC icon_title_gc;		       /* icon title background */
    
    GC menu_title_gc;		       /* menu titles */
    GC menu_entry_gc;		       /* menu entries */
    GC select_menu_gc;		       /* selected menu entries */
    GC disabled_menu_entry_gc;	       /* disabled menu entries */

    GC frame_gc;		       /* gc for resize/move frame (root) */
    GC line_gc;			       /* gc for drawing XORed lines (root) */
    GC copy_gc;			       /* gc for XCopyArea() */
    
    struct WPixmap *b_pixmaps[PRED_BPIXMAPS]; /* internal pixmaps for buttons*/

    struct WPixmap *button_arrow;       /* arrow pixmap for panel buttons */
    int app_menu_x, app_menu_y;	       /* position for application menus */

    struct WMenu *root_menu;	       /* root window menu */
    
    Window geometry_display;	       /* displays the geometry during 
					* window resize, move etc. */
    struct WImage *icon_back_image;   /* background image for icons */
    Pixmap icon_back_pixmap;	       /* same as above, but as pixmap */
    int icon_back_depth;	       /* depth of icon_back_pixmap */
    Pixmap default_icon;	       /* default pixmap for icons */
    Pixmap default_titled_icon;	       /* default pixmap for titled icons */
    
    /* state and other informations */
    short cascade_index;	       /* for cascade window placement */
    struct {
	unsigned int startup:1;	       /* during window manager startup */
    } flags;
} WScreen;


WScreen *wScreenInit(int screen_number);
void wScreenRemakeStackList(WScreen *scr);
void wScreenCommitStacking(WScreen *scr);


#endif
