/* 
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "wconfig.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "GNUstep.h"
#include "version.h"
#include "screen.h"
#include "wcore.h"

extern *DisplayName;

#ifdef USECPP
void
putdef(char *line, char *name, char *value)
{
    strcat(line, name);
    strcat(line, value);
}



void
putidef(char *line, char *name, int value)
{
    char tmp[64];
    sprintf(tmp, "%i", value);
    strcat(line, name);
    strcat(line, tmp);
}

       
char *
MakeCPPArgs(char *path)
{
    int i;
    char buffer[MAXLINE], *buf, *line;
    Visual *visual;
    
    line = wmalloc(MAXLINE);
    *line = 0;
    i=1;
    if (gethostname(buffer, MAXLINE)==0) {
	putdef(line, " -DHOSTNAME=", buffer);
    }
    putdef(line, " -DUSER=", getlogin());
    putidef(line, " -DUID=", getuid());
    buf = XDisplayName(DisplayString(dpy));
    putdef(line, " -DDISPLAY=", buf);
    putidef(line, " -DMAJ_VERSION=", MAJ_VERSION);
    putidef(line, " -DMIN_VERSION=", MIN_VERSION);
    
    
    visual = DefaultVisual(dpy, DefaultScreen(dpy));
    putidef(line, " -DVISUAL=", visual->class);
    
    putidef(line, " -DDEPTH=", DefaultDepth(dpy, DefaultScreen(dpy)));

    putidef(line, " -DSCR_WIDTH=", WidthOfScreen(DefaultScreenOfDisplay(dpy)));
    putidef(line, " -DSCR_HEIGHT=", 
	    HeightOfScreen(DefaultScreenOfDisplay(dpy)));

    strcpy(buffer, path);    
    buf = strrchr(buffer, '/');
    if (buf) *buf = 0; /* trunc filename */
    putdef(line, " -I", buffer);
    
#ifdef XPM
    strcat(line, " -DXPM");
#endif
#ifdef DITHERING
    strcat(line, " -DDITHERING");
#endif
    
#undef arg
#ifdef DEBUG
    puts("CPP ARGS");
    puts(line);
#endif
    return line;
}
#endif /* USECPP */




int
LevelToIndex(int level)
{
    switch (level) {
     case NSNormalWindowLevel:
	return 0;
     case NSFloatingWindowLevel:
	return 1;
     case NSDockWindowLevel:
	return 2;
     case NSSubmenuWindowLevel:
	return 3;
     case NSMainMenuWindowLevel:
	return 4;
     default:
	return 0;
    }
}



/*
 *----------------------------------------------------------------------
 * AddToStackList--
 * 	Inserts the frame in the top of the stacking list. The
 * stacking precedence (window levels and on_top) is kept intact. 
 * 
 * Returns:
 * 	None
 * 
 * Side effects:
 * 	The frame is added to it's screen's window list.
 *---------------------------------------------------------------------- 
 */
void
AddToStackList(WCoreWindow *frame)
{
    WCoreWindow *prev, *tmpw;
    int index = LevelToIndex(frame->stacking->window_level);

    frame->screen_ptr->window_count++;
    frame->screen_ptr->window_level_count[index]++;
    tmpw = frame->screen_ptr->stacking_list[index];
    if (!tmpw) {
	frame->screen_ptr->stacking_list[index] = frame;
	frame->stacking->above = NULL;
	frame->stacking->under = NULL;
	wScreenCommitStacking(frame->screen_ptr);	
	return;
    }
    prev = tmpw;

    if (!frame->stacking->on_top) {
	/* must put this window below other on top windows */	
	while (tmpw && tmpw->stacking->on_top) {
	    prev = tmpw;
	    tmpw = tmpw->stacking->under;
	}
	/* add window after our place */
	if (tmpw == frame->screen_ptr->stacking_list[index]) {
	    /* last window on list (top) */
	    tmpw->stacking->above = frame;
	    frame->stacking->under = tmpw;
	    frame->stacking->above = NULL;
	    frame->screen_ptr->stacking_list[index] = frame;
	} else if (tmpw == NULL) {
	    /* first window on list (bottom) */
	    prev->stacking->under = frame;
	    frame->stacking->under = NULL;
	    frame->stacking->above = prev;
	} else {
	    /* other cases */
	    frame->stacking->above = tmpw->stacking->above;
	    frame->stacking->under = tmpw;
	    tmpw->stacking->above->stacking->under = frame;
	    tmpw->stacking->above = frame;
	}
    } else {
	/* put on top of the stacking list */
	frame->stacking->above = NULL;
	frame->stacking->under = tmpw;
	tmpw->stacking->above = frame;
	frame->screen_ptr->stacking_list[index] = frame;
    }
    wScreenCommitStacking(frame->screen_ptr);
}

