/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "wconfig.h"

#include <X11/Xlib.h>

#include <stdlib.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "menu.h"
#include "window.h"
#include "application.h"
#include "appmenu.h"

/*
 * wApplication descriptors can be used both to track the list of
 * windows in a group for simple applications and to record information
 * for more complex applications with app defined menus and others.
 */


/* table of application instances */
static HashTable wApplicationTable;



void
wApplicationInitialize()
{
    table_init(&wApplicationTable);
}


/*
static WMenu *
createStandardMenu(WApplication *wapp)
{
    WMenu *menu, *windows;
    
    menu = wMenuCreate(wapp->main_window->screen_ptr, 
		       wapp->main_window->title);
    windows = wMenuCreate(wapp->main_window->screen_ptr, 
			  "Windows");
    wMenuAddCascade(menu, "Windows", windows);
    return menu;
}

*/

WApplication *
wApplicationCreate()
{
    WApplication *wapp;
    
    wapp = wmalloc(sizeof(WApplication));
    wapp->main_window = NULL;
    wapp->subwindows = NULL;
    wapp->menu = NULL;
    wapp->flags.hidden = 0;
    wapp->flags.dock_icon = 0;
    return wapp;
}

/*
 *---------------------------------------------------------------------- 
 * wApplicationAddWindowFor--
 * 	Finds the application descriptor for the given main window
 * and inserts subwindow in the list of subwidows for the application.
 * If the descriptor is not found, a new descriptor is created and 
 * returned.
 * 
 * Returns:
 * 	The application descriptor which main window is mainWindow.
 * 
 * Side effects:
 * 	A new application descriptor may be created and subwindow is
 * added in the descriptor.
 *---------------------------------------------------------------------- 
 */
WApplication *
wApplicationAddWindowFor(Window mainWindow, WWindow *subwindow)
{
    WApplication *wapp;
    
    wapp = table_iget(&wApplicationTable, (int)mainWindow);
    if (!wapp) {
#ifdef DEBUG
	puts("Creating new application group");
#endif	
	wapp = wApplicationCreate();
	table_iput(&wApplicationTable, mainWindow, wapp);
	wapp->main_window_id = mainWindow;
	/* while the real main window is not added, 
	 * use this one as the main window */
	wapp->main_window = subwindow;
	wapp->menu = NULL;
	wapp->flags.hidden = 0;
	wapp->flags.dock_icon = 0;
    }
#ifdef DEBUG
    if (subwindow->title)
      printf("Adding window %s (%x) to group\n",subwindow->title,
	     (unsigned)subwindow->client_win);
#endif
    wapp->subwindows = list_cons(subwindow, wapp->subwindows);
    if (subwindow->client_win == mainWindow) {
	wapp->main_window = subwindow;
	wapp->main_window_id = mainWindow;
	wapp->menu = wAppMenuGet(subwindow->screen_ptr, subwindow);
    }
    return wapp;
}


/*
 *----------------------------------------------------------------------
 * wApplicationRemoveWindow--
 * 	Removes a window from it's application descriptor.
 * 
 * Side effects:
 * 	If the window to be removed is the main window, the whole
 * application descriptor will be destroyed and the belongs_to field
 * of all subwindows will be set to NULL.
 *---------------------------------------------------------------------- 
 */
void
wApplicationRemoveWindow(WApplication *wapp, WWindow *window)
{    
#ifdef DEBUG    
    printf("Removing Window %x from app group\n",(unsigned)window->client_win);
#endif    
    wapp->subwindows = list_remove_elem(wapp->subwindows, window);
    if (wapp->subwindows==NULL) {
#ifdef DEBUG	
	puts("destroying application group");
#endif
	table_idelete(&wApplicationTable, wapp->main_window_id);
	wApplicationDestroy(wapp);
    } else if (wapp->main_window == window) {
	/* make some other window the main window */
	wapp->main_window = wapp->subwindows->head;
    }

}



void
wApplicationDestroy(WApplication *wapp)
{
    LinkedList *list= wapp->subwindows;
    WWindow *wwin;
 
    while (list) {
	wwin = list->head;
	if (wwin->belongs_to == wapp)
	  wwin->belongs_to = NULL;
	list = list->tail;
    }
    if (wapp->menu)
      wMenuDestroy(wapp->menu);
    wfree(wapp);
}


