/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "wconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "window.h"
#include "funcs.h"
#include "menu.h"
#include "dialog.h"
#include "resource.h"
#include "application.h"
#include "keybind.h"
#ifdef DYNSETUP
#include "setup.h"
#endif
#include "version.h"

#ifdef SOUNDS
#include "sound.h"
#endif

/****** Global Variables ******/

/* general info */

Display	*dpy;
/* the root window */
char 	*ProgName;

WScreen wScreen;


WPreferences wPreferences;

WResource wResourceDefault;


WShortKey *wKeyBindings[WKBD_LAST];


/* modules */
int ModuleNo;

/* XContexts */
XContext wWinContext;
XContext wStackContext;

/* Atoms */
Atom _XA_WM_STATE;
Atom _XA_WM_CHANGE_STATE;
Atom _XA_WM_PROTOCOLS;
Atom _XA_WM_TAKE_FOCUS;
Atom _XA_WM_DELETE_WINDOW;
Atom _XA_WM_SAVE_YOURSELF;
Atom _XA_WM_CLIENT_LEADER;
Atom _XA_WM_COLORMAP_WINDOWS;

Atom _XA_GNUSTEP_WM_ATTR;

Atom _XA_WINDOWMAKER_MENU;
Atom _XA_WINDOWMAKER_WM_PROTOCOLS;


/* cursors */
Cursor wCursor[WCUR_LAST];

/* last event timestamp for XSetInputFocus */
Time LastTimestamp;

/* paths */
char *MenuConfigPath=NULL;
char *PixmapPath=NULL;


#ifdef SHAPE
int ShapeEventBase;
#endif

/******** End Global Variables *****/

static char **Arguments;
static int ArgCount;

extern void EventLoop();
extern void StartUp();

void
Restart(char *manager)
{
    char *prog=NULL;
    char *argv[MAX_RESTART_ARGS];
    int i;
#ifdef DYNSETUP
    wSetupClose(&wScreen);
#endif
    if (manager && manager[0]!=0) {
	prog = argv[0] = strtok(manager, " ");
	for (i=1; i<MAX_RESTART_ARGS; i++) {
	    argv[i]=strtok(NULL, " ");
	    if (argv[i]==NULL) {
		break;
	    }
	}
    }

    XCloseDisplay(dpy);
    if (!prog)
      execvp(Arguments[0], Arguments);
    else {
	execvp(prog, argv);
	/* fallback */
	execv(Arguments[0], Arguments);
    }
    wFatal("Restart failed!!!");
    exit(-1);
}

/*
 *---------------------------------------------------------------------
 * wAbort--
 * 	Do a major cleanup and exit the program
 * 
 *---------------------------------------------------------------------- 
 */
void
wAbort()
{
    RestoreDesktop(&wScreen);
    printf("%s aborted.", ProgName);
    exit(1);
}


void
print_help()
{
    printf("usage: %s [-options]", ProgName);
    puts("options:");
#ifdef USECPP
    puts(" -nocpp 		disable preprocessing of configuration files");
#endif
    puts(" -display host:dpy	display to use");
    puts(" -version		print version and exit");
}

int
main(int argc, char **argv)
{
    int i, restart=0;
    char *display_name="";
    char *tmp;
#ifdef I18N
    char *locale;
#endif

    ArgCount = argc;
    Arguments = argv;

    ProgName = strrchr(argv[0],'/');
    if (!ProgName)
      ProgName = argv[0];
    else
      ProgName++;

    restart = 0;
    
    memset(&wPreferences, 0, sizeof(WPreferences));
    
    if (argc>1) {
	for (i=1; i<argc; i++) {
#ifdef USECPP
	    if (strcmp(argv[i], "-nocpp")==0) {
		wPreferences.flags.nocpp=1;
	    } else
#endif
	    if (strcmp(argv[i], "-version")==0) {
		printf("WindowMaker %i.%i.%i\n", MAJ_VERSION, MIN_VERSION, 
		       SUBMIN_VERSION);
		exit(0);
	    } else if (strcmp(argv[i], "-display")==0) {
		display_name = argv[++i];
	    } else {
		print_help();
		exit(0);
	    }
	}
    }
#ifdef I18N
    locale = getenv("LANG");
    if (!setlocale(LC_CTYPE, locale)) {
	wWarning("could not set locale");
    }
#endif
    MenuConfigPath = find_file(DEF_CONFIG_PATHS, DEF_MENUCONFIG);

    /* pixmap path can be freed later, if the user specified a new
     * path in configuration */
    PixmapPath = strdup(DEF_PIXMAP_PATHS);

    /* open display */
    dpy = XOpenDisplay(display_name);
    if (dpy == NULL) {
	wFatal("could not open display %s", XDisplayName(display_name));
	exit(1);
    }
#ifdef DEBUG
    XSynchronize(dpy, True);
#endif

    wKeyboardInitialize();
    wResourceInitialize();
    wApplicationInitialize();
    wWindowInitialize();
    wMenuInitialize();
    wDialogInitialize();
#ifdef SOUNDS
    wSoundInitialize();
#endif
    StartUp(display_name);

    if (0 && !restart) {
	pid_t pid;

	tmp = find_file(DEF_CONFIG_PATHS, DEF_STARTUP_SCRIPT);
	if (tmp) {
	    pid=fork();
	    
	    if (pid==0) {
		system(tmp);
		exit(0);
	    }
	}
    }

    EventLoop();
    return -1;
}
