/*
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef WMMENU_H_
#define WMMENU_H_

#include "wcore.h"
#include "button.h"

typedef struct WMenuEntry {
    char *text;			       /* entry text */
    int text_width;		       /* width of the text (pixels) */
    char *rtext;		       /* text to show in the right part */
    void (*callback)(struct WMenu *menu, struct WMenuEntry *entry);
    void *clientdata;		       /* data to pass to callback */
    struct WMenu *cascade;	       /* cascade menu */
    short order;
    struct {
	unsigned int enabled:1;	       /* entry is selectable */
	unsigned int selected:1;       /* entry is selected */
    } flags;
} WMenuEntry;


typedef struct WMenu {
    struct WMenu *parent;

    WScreen *screen_ptr;	       /* pointer to the screen */
    /* decorations */
    WCoreWindow *frame;		       /* the frame window */
    WCoreWindow *menu;		       /* the window menu */
    WCoreWindow *titlebar;	       /* the titlebar */
    WTexData titlebar_texture_data;
    WTexData menu_texture_data;
    WButton *close_button;
    int frame_x, frame_y;	       /* position of the frame in root*/

    char *title;		       /* window name (title) */
    int title_width;		       /* width of the title in pixels */
    WMenuEntry **entries;	       /* array of entries */
    short entry_no;		       /* number of entries */
    short alloced_entries;	       /* number of entries allocated in 
					* entry array */
    short selected_entry;
    
    short entry_height;		       /* height of each entry */
    
    struct {
	unsigned int realized:1;       /* whether the window was configured */
	unsigned int app_menu:1;       /* this is a application or root menu */
	unsigned int mapped:1;	       /* if menu is already mapped on screen*/
	unsigned int buttoned:1;       /* if the close button is visible
					* (menu was torn off) */
    } flags;
} WMenu;

void wMenuInitialize();
void wMenuPaint(WMenu *menu);
void wMenuDestroy(WMenu *menu);
void wMenuRealize(WMenu *menu);
WMenuEntry *wMenuInsertCascade(WMenu *menu, int index, char *text, 
			       WMenu *cascade);
#ifdef notdef
void wMenuInsertItem(WMenu *menu, int index, char *text, int code);
#endif
WMenuEntry *wMenuInsertCallback(WMenu *menu, int index, char *text, 
				void (*callback)(WMenu *menu, WMenuEntry *entry),
				void *clientdata);

#define wMenuAddCascade(menu, text, cascade) \
	wMenuInsertCascade(menu, -1, text, cascade)
#ifdef notdef
#define wMenuAddItem(menu, text, code) \
	wMenuInsertItem(menu, -1, text, code)
#endif
#define wMenuAddCallback(menu, text, callback, data) \
	wMenuInsertCallback(menu, -1, text, callback, data)

void wMenuRemoveItem(WMenu *menu, int index);

WMenu *wMenuCreate(WScreen *screen, char *title, int main_menu);
void wMenuMap(WMenu *menu);
void wMenuMapAt(WMenu *menu, int x, int y);
void wMenuUnmap(WMenu *menu);
void wMenuSetEnabled(WMenu *menu, int index, int enable);



#endif
