/* 
 * savews - saves workspace state
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

Display *dpy;
char *host;

void put_command(Window fwin)
{
    int x, y;
    unsigned int w, h, g, b;
    XTextProperty tp;
    int i;
    static int first = 1;
    XSizeHints hints;
    long rets;
    Window win, r;
    char **argv;
    int argc;

    
    win = XmuClientWindow(dpy, fwin);
    if (win==None) return;


    if (!XGetGeometry(dpy, win, &r, &x, &y, &w, &h, &b, &b)) 
      return;
    if (!XGetGeometry(dpy, fwin, &r, &x, &y, &b, &b, &b, &b)) 
      return;
    XGetWMClientMachine(dpy, win, &tp);
    if (tp.value==NULL || tp.encoding!=XA_STRING || tp.format !=8)
      return;
    if (strcmp(tp.value, host)!=0)
      return;

    if (!XGetCommand(dpy, win, &argv, &argc))
      return;    
    XGetWMNormalHints(dpy, win, &hints, &rets);
    if (rets & PResizeInc) {
	if (rets & PBaseSize) {
	    w -= hints.base_width;
	    h -= hints.base_height;
	}
	if (hints.width_inc<=0)
	  hints.width_inc=1;
	if (hints.height_inc<=0)
	  hints.height_inc=1;
	w = w / hints.width_inc;
	h = h / hints.height_inc;
    }
    if (!first) {
	puts("&");
    }
    first = 0;
    for (i=0; i<argc; i++) {
	if (strcmp(argv[i], "-geometry")==0) {
	    if (i<=argc-1) {
		i++;
	    }
	} else {
	    printf("%s ", argv[i]);
	}
    }
    printf("-geometry %dx%d+%d+%d ", w, h, x, y);
    XFreeStringList(argv);
}

int main(int argc, char **argv)
{
    Window *wins;
    int nwins, i;
    Window foo;

    host = getenv("HOST");
    if (!host) {
	printf("%s: could not get HOST environment variable\n", argv[0]);
    }
    dpy = XOpenDisplay("");
    if (!dpy) {
	printf("%s: could not open display\n", argv[0]);
	exit(1);
    }
    if (!XQueryTree(dpy, DefaultRootWindow(dpy), &foo, &foo, &wins, &nwins)) {
	printf("%s: XQueryTree() failed\n", argv[0]);
	exit(1);
    }
    for (i=0; i<nwins; i++) {
	put_command(wins[i]);
    }
    puts("");
    exit(0);
}
