/* dicon.c- individual Dock icon
 * 
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "wconfig.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "wcore.h"
#include "window.h"
#include "pixmap.h"
#include "dicon.h"


/*
 * TODO:
 *  how to find out which running instance was launched from where
 * (command line, dock etc.)
 */

/**** Global variables ****/
extern char *PixmapPath;

static void iconMouseDown(WObjDescriptor *desc, XEvent *event);
static void iconDblClick(WObjDescriptor *desc, XEvent *event);

WMethodTable wDockIconMethods = {
    (WEventHandler)iconMouseDown,
      NULL,
      (WEventHandler)iconDblClick,
      NULL
};


/*
 *---------------------------------------------------------------------- 
 * wDockIconCreateFromIcon--
 * 	Create an icon for use in the Dock, by "converting" a normal
 * icon to a dock icon. Tries to guess how the app is launched and
 * if it can't, fail.
 * 
 * Returns:
 * 	A WDockIcon structure or NULL, if the icon can't be docked.
 * 
 * Side effects:
 *	The dock_icon flag in the icon is set.
 * 
 * Notes:
 * 	WIcons with the dock_icon flag set should not be destroyed
 * by other routines.
 * 	Only GNUstep apps can be hidden into the Dock icon, since
 * we can have multiple instances of X apps running.
 *---------------------------------------------------------------------- 
 */
WDockIcon*
wDockIconCreateFromIcon(WIcon *icon)
{
    WDockIcon *dicon;
    Window main_win;
    char **argv;
    int argc;

    dicon = wmalloc(sizeof(WDockIcon));
    memset(dicon, 0, sizeof(WDockIcon));
    dicon->icon = icon;
    /* we've got to find out how the app is invoked:
     * by getting the path for itself that's passed
     * by the app, using the icon in the .App wrapper
     * or
     * with WM_COMMAND, using icon_file as it's icon
     * or
     * if none of the above is possible, return NULL */
    
    
    /* check WM_COMMAND */
    if (!wApplicationOf(icon->owner->main_window)) {
	main_win = icon->owner->client_win;
    }  else {
	main_win = icon->owner->main_win;
    }
    if (!XGetCommand(dpy, main_win, &argv, &argc)) {
	wfree(dicon);
	return NULL;
    }
    dicon->icon_file = icon->owner->icon_file;
    /* TODO: finish argv -> command conversion */
    dicon->command = strdup(argv[0]);

    XFreeStringList(argv);

    icon->dock_icon = True;
    return dicon;
}


/*
WDockIcon*
wDockIconCreateForApplication(WScreen *scr, char *appName)
{
    
}
*/

WDockIcon*
wDockIconCreateForCommand(WScreen *scr, char *command, char *image)
{
    
}


