/* 
 *  WindowMaker window manager
 * 
 *  Copyright (c) 1997 Shige Abe and 
 *		       Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "wconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "generic/wwmlib.h"
#include "WindowMaker.h"
#include "window.h"
#include "actions.h"
#include "client.h"
#include "funcs.h"
#include "stacking.h"


/********* Global Variables *******/
extern WScreen wScreen;



/*
 * TODO:
 * 
 * - disable entry of the current selected window
 * + update window title when it's changed
 *
 */


/*
 * FocusWindow
 *
 *  - Needs to check if already in the right workspace before
 *    calling wChangeWorkspace?
 *
 *  Order:
 *    Switch to correct workspace
 *    Unshade if shaded
 *    If iconified then deiconify else focus/raise.
 */
static void
focusWindow(WMenu *menu, WMenuEntry *entry)
{
    WWindow *w;
    w = (WWindow*)entry->clientdata;
    wChangeWorkspace(menu->screen_ptr,w->workspace);
    if (w->flags.shaded) {
	wUnshadeWindow(w);
    }
    if (w->flags.hidden) {
	wUnhideApplication(wApplicationOf(w->main_window));
    } else if (w->flags.miniaturized) {
	wDeiconifyWindow(w);
    } else {
	wSetFocusTo(menu->screen_ptr,w);
	wRaiseFrame(w->frame);
    }
}
 
/*
 *
 * Open switch menu 
 *
 */
void OpenSwitchMenu(WScreen *scr, int x, int y)
{
    WMenu *switchmenu = scr->switch_menu;
    WWindow *wwin;
    char title[MAX_MENU_TEXT_LENGTH+6];

    return;
    if (switchmenu) {
	if (switchmenu->flags.mapped) {
	    if (!switchmenu->flags.buttoned)
	      wMenuUnmap(switchmenu);
	    else
	      wRaiseFrame(switchmenu->frame);
	} else {
	    wMenuMapAt(switchmenu, x-switchmenu->frame->width/2,
		       y-switchmenu->titlebar->height/2);
	}
	return;
    }
    switchmenu = wMenuCreate(scr,"Windows",True);
    wwin = wScreen.focused_window;
    while (wwin) {
	WMenuEntry *entry;
	char *wspace;

	sprintf(title, "%s", wwin->title);
	entry = wMenuAddCallback(switchmenu, title, focusWindow, wwin);
	if (!wwin->window_flags.focusable)
	  wMenuSetEnabled(switchmenu, entry->order, False);
	wspace = malloc(20);
	if (wspace) {
	    sprintf(wspace, "[%i]", wwin->workspace+1);
	    entry->rtext = wspace;
	}
        wwin = wwin->prev;
    }
    scr->switch_menu = switchmenu;
    if (switchmenu) {
	if (!switchmenu->flags.realized)
	  wMenuRealize(switchmenu);
	wMenuMapAt(switchmenu, x-switchmenu->frame->width/2,
		   y-switchmenu->titlebar->height/2);
    }
}

/*
 *
 * Update switch menu
 *
 * TODO: Optimization
 *
 */
void UpdateSwitchMenu(WScreen *scr)
{
    WMenu *switchmenu = scr->switch_menu;
    WWindow *wwin;
    char title[MAX_MENU_TEXT_LENGTH+6];
    int i,j;

    /*
     *  This menu is updated under the following conditions:
     *
     *    1.  When a window is created.
     *    2.  When a window is destroyed.
     *
     *  The easiest way to deal with both situations is to
     *  just redo the menu from scratch.  It's both easier
     *  to code and since window lists are short, it doesn't
     *  take much time.
     *
     *  Feel free to delete this after you've read it.
     */

    j = switchmenu->entry_no;

    /*
     *  I'm trying to clear all menu entries with this
     *  loop.  Is there a better/faster way to do it?
     *  - Remove only the needed one.
     */
    for (i=0; i<j; i++)
         wMenuRemoveItem(switchmenu,switchmenu->entry_no-1);

    wwin = wScreen.focused_window;
    while (wwin) {
        WMenuEntry *entry;
        char *wspace;

        sprintf(title, "%s", wwin->title);
        entry = wMenuAddCallback(switchmenu, title, focusWindow, wwin);
	if (!wwin->window_flags.focusable)
	  wMenuSetEnabled(switchmenu, entry->order, False);
        wspace = malloc(20);
        if (wspace) {
            sprintf(wspace, "[%i]", wwin->workspace+1);
            entry->rtext = wspace;
        }
	
        wwin = wwin->prev;
    }
    scr->switch_menu = switchmenu;
    wMenuRealize(switchmenu);
    wMenuPaint(switchmenu);
}
