/*
 *  UserDefaults database access for WindowMaker
 * 
 *  Copyright (c) 1997 Alfredo K. Kojima
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * partial implementation of the NSUserDefaults.m class from GNUstep base
 * for C.
 *
 * 
 * Have no idea about file format for persistent domains.
 * I'm using this until I figure GNUstep's:
 * 
 * KeyString ValueString\n
 * 
 */
#ifndef WMUSERDEFS_H_
#define WMUSERDEFS_H_

#include "wwmlib.h"

typedef struct WUserDefaults {
    HashTable *pers_domains;	       /* persistent domains */
    HashTable *volat_domains;	       /* volatile domains */
    char **search_list;
} WUserDefaults;

/* standard NS domains */
extern char *NSArgumentDomain;
extern char *NSGlobalDomain;
extern char *NSRegistrationDomain;


/* initialization */
WUserDefaults* wUDInit();
WUserDefaults* wUDInitWithUser(char *username);
WUserDefaults* wUDInitWithFile(char *file);

/* get data */
int wUDBoolForKey(WUserDefaults *udb, char *defname);
int wUDIntegerForKey(WUserDefaults *udb, char *defname);
char* wUDStringForKey(WUserDefaults *udb, char *defname);
/* set data */
void wUDSetBool(WUserDefaults *udb, char *defname, int value);
void wUDSetInteger(WUserDefaults *udb, char *defname, int value);
void wUDSetString(WUserDefaults *udb, char *defname, int value);

/* persistent domains */
HashTable* wUDPersistentDomainForName(WUserDefaults *udb, char *domain_name);
char** wUDPersistentDomainNames(WUserDefaults *udb);
void wUDRemovePersistentDomainForName(WUserDefaults *udb, char *domain_name);
void wUDSetPersistentDomain(WUserDefaults *udb, HashTable *domain, 
			   char *domain_name);
void wUDSynchronize(WUserDefaults *udb);

/* volatile domains */
HashTable* wUDVolatileDomainForName(WUserDefaults *udb, char *domain_name);
char** wUDVolatileDomainNames(WUserDefaults *udb);
void wUDRemoveVolatileDomainForName(WUserDefaults *udb, char *domain_name);
void wUDSetVolatileDomain(WUserDefaults *udb, HashTable *domain, 
			   char *domain_name);

#endif /* WMUSERDEFS_H_ */
