/*
 * GNUStep application hints
 * 
 * Copyright (c) 1996 Alfredo K. Kojima
 */

#include "../configure.h"

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include "afterstep.h"
#include "wmgnustep.h"
#include "misc.h"
#include "screen.h"

#ifdef GNUSTEP_HINTS

extern Atom _XA_GNUSTEP_WM_HINTS;

void GetGNUStepHints(ASWindow *t)
{
    int actual_format;
    Atom actual_type;
    unsigned long nitems, bytes_after;

    if (XGetWindowProperty (dpy, t->w, _XA_GNUSTEP_WM_HINTS, 0L, 
			    sizeof(PropGNUStepHints), False,
			    _XA_GNUSTEP_WM_HINTS, &actual_type, &actual_format, 
			    &nitems, &bytes_after,
			    (unsigned char **)&t->gstep_hints)==Success) {
	if(nitems >= sizeof(PropGNUStepHints)/sizeof(CARD32)) {
	    return;
	} 
    }
    t->gstep_hints = NULL;
}

/*
 * Setup window according to the gnustep hints
 */
void SelectGNUStepDecor(ASWindow *t, unsigned long tflags, int border_width, 
			int resize_width)
{
    int decor,i;
    PropGNUStepHints *prop;

    if(!(tflags & BW_FLAG))
      border_width = Scr.NoBoundaryWidth;
    
    if(!(tflags & NOBW_FLAG))
      resize_width = Scr.BoundaryWidth;
    
    for(i=0;i<5;i++) {
	t->left_w[i] = 1;
	t->right_w[i] = 1;
    }

    if(t->gstep_hints)  {
	prop = (PropGNUStepHints *)t->gstep_hints;
	decor = prop->decorations;
	t->functions = prop->functions;
    } else {
	SelectDecor(t,tflags,border_width,resize_width);
	return;
    }
    /* basically the same functionality of MWM */
    if (t->functions & GNUSTEP_FUNC_ALL) {
      t->functions &= ~GNUSTEP_FUNC_ALL;
      t->functions = (GNUSTEP_FUNC_RESIZE | GNUSTEP_FUNC_MOVE | 
		      GNUSTEP_FUNC_MINIMIZE | GNUSTEP_FUNC_MAXIMIZE |
		      GNUSTEP_FUNC_CLOSE) & ~(t->functions);
    }

    if (decor & GNUSTEP_DECOR_ALL) {
      decor &= ~GNUSTEP_DECOR_ALL;
      decor = (GNUSTEP_DECOR_TITLEBAR | GNUSTEP_DECOR_HANDLE |
	       GNUSTEP_DECOR_SIZE_BUTTON | GNUSTEP_DECOR_CLOSE_BUTTON) 
	  & ~(decor);
	/* these are treated differently */
	if (prop->decorations & GNUSTEP_DECOR_SIZE_MAXIMIZE) {
	    decor |= GNUSTEP_DECOR_SIZE_MAXIMIZE;
	} else {
	    decor &= ~GNUSTEP_DECOR_SIZE_MAXIMIZE;
	}
	
	if (prop->decorations & GNUSTEP_DECOR_CLOSE_BROKEN) {
	    decor |= GNUSTEP_DECOR_CLOSE_BROKEN;
	} else {
	    decor &= ~GNUSTEP_DECOR_CLOSE_BROKEN;
	}	
    }
    
  /* Selected the mwm-decor field, now trim down, based on
   * .steprc entries */
  if ((tflags & NOTITLE_FLAG)
     || ((!(Scr.flags & DecorateTransients)) && (t->flags & TRANSIENT)))
    decor &= ~GNUSTEP_DECOR_TITLEBAR;

  if ((tflags & NOBORDER_FLAG)
     || ((!(Scr.flags&DecorateTransients)) && (t->flags & TRANSIENT)))
    decor &= ~GNUSTEP_DECOR_HANDLE;

#ifdef SHAPE
    if(t->wShaped)
      decor &= ~GNUSTEP_DECOR_HANDLE;
#endif
    /* Assume no decorations, and build up */
    t->boundary_width = 0;
    t->boundary_height = 0;
    t->corner_width = 0;
    t->title_height = 0;
    t->button_height = 0;

    if (decor & GNUSTEP_DECOR_TITLEBAR)  {
	/*  A title barm with no buttons in it
	 * window gets a 1 pixel wide black border. */
	t->flags |= TITLE;
	t->title_height = Scr.TitleHeight;
    }
    if(decor & GNUSTEP_DECOR_HANDLE) {
	/* A wide border, with corner tiles is desplayed
	 * (10 pixels - 2 relief, 2 shadow) */
	t->flags |= BORDER;
    } else {
	t->flags &= ~BORDER;	
    }
    if((decor & GNUSTEP_DECOR_CLOSE_BUTTON)==0)  {
	/* title-bar + iconify button, no menu button.
	 * window gets 1 pixel wide black border */
	t->right_w[0] = None;
    }
    if((decor & GNUSTEP_DECOR_SIZE_BUTTON)==0) {
	/* title-bar + maximize button, no menu button, no iconify.
	 * window has 1 pixel wide black border */
	t->left_w[0] = None;
    }
    t->boundary_width = 0;
    t->boundary_height = (t->flags & BORDER)? 
      Scr.BoundaryWidth : Scr.NoBoundaryWidth;
    t->corner_width = Scr.TitleHeight + t->boundary_height; 

    /*
    if (t->buttons & BUTTON1) t->left_w[0]=None;
    */
    t->left_w[1]=None;
    t->left_w[2]=None;
    t->left_w[3]=None;
    t->left_w[4]=None;
    /*
    if (t->buttons & BUTTON2) t->right_w[0]=None;
     */
    t->right_w[1]=None;
    t->right_w[2]=None;
    t->right_w[3]=None;
    t->right_w[4]=None;
    
    t->nr_left_buttons = Scr.nr_left_buttons;
    t->nr_right_buttons = Scr.nr_right_buttons;
    
    for(i=0;i<Scr.nr_left_buttons;i++) {
	if(t->left_w[i] == None)
	  t->nr_left_buttons--;
    }
    if (t->nr_left_buttons>1) t->nr_left_buttons=1;
    for(i=0;i<Scr.nr_right_buttons;i++) {
	if(t->right_w[i] == None)
	  t->nr_right_buttons--;
    }
    if (t->nr_right_buttons>1) t->nr_right_buttons=1;
    t->bw = 1;
    if(t->title_height > 0)
      t->title_height += t->bw;
    if(t->boundary_height == 0)
      t->flags &= ~BORDER;
    t->button_height = t->title_height- 7;
}

#endif


