/*
 * ScrolledCanvas.h -- Header for the ScrolledCanvas widget
 *
 * This file is part of the libXas project.
 * Copyright (c) 1997 Sbastien Phlep (phelep@iu-vannes.fr)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _SCROLLED_CANVAS_H_
#define _SCROLLED_CANVAS_H_

/* Define the ScrolledCanvas widget structure */
typedef struct {
  /* Windows */
  Window	bbox_w;		/* Bounding box (effective canvas) */
  Window	cnvs_w;		/* Real canvas */
  Window	chld_w;		/* Window to show in the canvas */
  Window	vsbb_w;		/* Vertical slider's bounding box */
  Window	vubt_w;		/* Vertical up arrow button */
  Window	vdbt_w;		/* Vertical down arrow button */
  Window	vsld_w;		/* Vertical sliding button */
  Window	hsbb_w;		/* Horizontal slider's bounding box */
  Window	hlbt_w;		/* Horizontal left arrow button */
  Window	hrbt_w;		/* Horizontal right arrow button */
  Window	hsld_w;		/* Horizontal sliding button */
  
  /* Graphical contexts */
  GC		bbox_c;		/* Bounding box (effective canvas) */
  GC		cnvs_c;		/* Canvas' bounding box */
  GC		vubt_c;		/* Vertical up arrow button */
  GC		vdbt_c;		/* Vertical down arrow button */
  GC		vsld_c;		/* Vertical sliding button */
  GC		hlbt_c;		/* Horizontal left arrow button */
  GC		hrbt_c;		/* Horizontal right arrow button */
  GC		hsld_c;		/* Horizontal sliding button */

  /* Methods saying how and when showing objects */
  Bool		enableVB;	/* True to display vertical scrolling buttons */
  Bool		enableHB;	/* True to display horizontal scrolling buttons */
  Bool		forceSB;	/* True to always display sliding buttons */
  Bool		enableSS;	/* True if you want the sliders to sunk when they are pressed */
} ScrolledCanvas;

/* Some definitions used by the ScrolledCanvas widget */
#define UNCHANGED -1

/* Prototypes for functions used in this module */
extern ScrolledCanvas XasAddScrolledCanvas(Window parent, Window child, int x, int y, int w, int h,
					   Bool enableVertButtons, Bool enableHorizButtons, Bool forceSlidingButtons, Bool enableSoftSliders);
extern int XasSetChildPositionInCanvas(ScrolledCanvas me, int x_coord, int y_coord);
extern void _XasManageSC_Events(ScrolledCanvas me, XEvent event);
extern void _XasMapSC_Windows(ScrolledCanvas me);

#endif	/* _SCROLLED_CANVAS_H_ */
