/*
 * GraphicalUtilities.c -- Simple graphical routines used by widgets
 *
 * This file is part of the libXas project.
 * Copyright (c) 1997 Sbastien Phlep (phelep@iu-vannes.fr)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <X11/Xlib.h>
#include <stdio.h>
#include "GraphicalUtilities.h"

/* Here are some global variables */
Pixel xasBlack, xasGrey, xasGrey44, xasGrey60, xasGrey90, xasAntqw3, xasWhite;
Display *xasDisplay;

/* Here we go ! */

extern Pixel
_XasGetPixel(char *name)
/*
 * This routine returns the Pixel value of the color
 * using its textual name 'name'.
 */
{
  XColor                color;
  Window                root_w;
  XWindowAttributes     attributes;
  char                  *myName="test";
  
  root_w=DefaultRootWindow(xasDisplay);
  XGetWindowAttributes(xasDisplay,root_w,&attributes);
  color.pixel = 0;
  if (!XParseColor (xasDisplay, attributes.colormap, name, &color)) 
    fprintf(stderr,"%s: Can't parse color %s\n",myName,name);
  else if(!XAllocColor (xasDisplay, attributes.colormap, &color)) 
    fprintf(stderr,"%s: Can't allocate color %s\n",myName,name);
  
  return color.pixel;
};


extern void
_XasDrawObjectBorder(Window box_w, GC box_c, int state)
/*
 * This routine gives an X-Window its border aspect.
 * It needs the 'box_w' object's X-Window, and the 'box_c' object's GC.
 * Border's aspect depends on the 'state' value:
 *  XAS_RELIEF_FTBOX: a simple black line, making the object look flat,
 *  XAS_RELIEF_DNBOX: two colors, making the object look pushed,
 *  XAS_RELIEF_UPBOX: two colors, making the object look released.
 */
{
  Window	tmp_w;
  int		tmp_x, tmp_y;
  unsigned int	tmp_l, tmp_h, tmp_b, tmp_d;
  Pixel		color1, color2;
  
  XGetGeometry(xasDisplay,box_w,&tmp_w,&tmp_x,&tmp_y,&tmp_l,&tmp_h,&tmp_b,&tmp_d);
  switch(state) {
  case XAS_RELIEF_UPBOX:
    color1 = xasGrey44;
    color2 = xasGrey90;
    break;
  case XAS_RELIEF_DNBOX:
    color1 = xasGrey90;
    color2 = xasGrey44;
    break;
  default:
  case XAS_RELIEF_FTBOX:
    color1 = xasBlack;
    color2 = xasBlack;
    break;
  }
  XSetForeground(xasDisplay,box_c,color1);
  XDrawLine(xasDisplay,box_w,box_c,0,(tmp_h-1),(tmp_l-1),(tmp_h-1));
  XDrawLine(xasDisplay,box_w,box_c,(tmp_l-1),(tmp_h-1),(tmp_l-1),0);
  XSetForeground(xasDisplay,box_c,color2);
  XDrawLine(xasDisplay,box_w,box_c,(tmp_l-2),0,0,0);
  XDrawLine(xasDisplay,box_w,box_c,0,0,0,(tmp_h-2));
};

