/*
 * ScrolledCanvasT.c -- Test program for the ScrolledCanvas widget
 *
 * This file is part of the libXas project.
 * Copyright (c) 1997 Sbastien Phlep (phelep@iu-vannes.fr)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <X11/Xlib.h>
#include <stdio.h>
#include "ScrolledCanvas.h"
#include "GraphicalUtilities.h"

void
main(int argc, char *argv[])
{
  Display		*dpy;
  Window		main_w, child_w;
  GC			main_c, child_c;
  ScrolledCanvas	cnvs;
  XEvent		event;
  Pixel			color[256];
  Bool			again;
  int			loop=0, i, red=16, green=16, blue=16;
  char			color_string[10];

  /* Open the display */
  dpy = XOpenDisplay(NULL);
  if(dpy == NULL) {
    fprintf(stderr,"Erreur avec la variable DISPLAY!\n");
    exit(1);
  }

  /* Set up the colors */
  xasDisplay = dpy;
  xasBlack  = _XasGetPixel("black");
  xasGrey44 = _XasGetPixel("grey44");
  xasGrey60 = _XasGetPixel("grey60");
  xasGrey   = _XasGetPixel("grey");
  xasGrey90 = _XasGetPixel("grey90");
  xasAntqw3 = _XasGetPixel("AntiqueWhite3");

  for(i=0; i<256; i++) {
    sprintf(color_string,"#%x%x%x",red,green,blue);
    color[i] = _XasGetPixel(color_string);
    red += 1; green +=1; blue +=1;
  }
  /* Create the main window */
  main_w = XCreateSimpleWindow(dpy,DefaultRootWindow(dpy),100,100,600,300,0,xasBlack,xasGrey);
  main_c = XCreateGC(dpy,main_w,0,NULL);
  XSelectInput(dpy,main_w,ButtonPressMask);

  /* Create the child window */
  child_w = XCreateSimpleWindow(dpy,main_w,10,10,1000,500,0,xasBlack,xasAntqw3);
  child_c = XCreateGC(dpy,child_w,0,NULL);

  /* Create the scrolled canvas */
  cnvs = XasAddScrolledCanvas(main_w,child_w,50,50,500,200,True,True,False,True);

  /* Map the windows */
  _XasMapSC_Windows(cnvs);
  XMapSubwindows(dpy,main_w);
  XMapRaised(dpy,main_w);

  /* Enter the event loop */
  again = True;
  while(again) {
    XNextEvent(dpy,&event);
    _XasManageSC_Events(cnvs,event);

    switch(event.type) {
    case Expose:
      if(event.xexpose.count == 0) {
	if(event.xexpose.window == child_w) {
	  for(i=0; i<256; i++) {
	    XSetForeground(dpy,child_c,color[i]);
	    XFillRectangle(dpy,child_w,child_c,((5*i)+10),10,5,5);
	  }
	    /*XSetForeground(dpy,child_c,xasBlack);
	  XFillArc(dpy,child_w,child_c,50,50,100,100,0,(360*64));*/
	}
      }
      break;
    case ButtonPress:
      if((event.xbutton.window == main_w) && (event.xbutton.button == 1)) {
	switch(loop) {
	case 0:
	  /*XResizeWindow(dpy,child_w,100,100);*/
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,10,10));
	  break;
	case 1:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,10,UNCHANGED));
	  /*XResizeWindow(dpy,child_w,10000,10000);*/
	  break;
	case 2:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,UNCHANGED,10));
	  /*XMoveWindow(dpy,child_w,-150,100);*/
	  break;
	case 3:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,UNCHANGED,UNCHANGED));
	  break;
	case 4:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,-100,UNCHANGED));
	  break;
	case 5:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,UNCHANGED,-100));
	  break;
	case 6:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,-100,-100));
	  break;
	case 7:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,600,100));
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,UNCHANGED,UNCHANGED));
	  break;
	case 8:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,UNCHANGED,600));
	  break;
	default:
	  printf("Pos: %d\n",XasSetChildPositionInCanvas(cnvs,0,0));
	  /*XResizeWindow(dpy,child_w,500,500);*/
	  break;
	}
	loop++;
      }
      break;
    default:
      break;
    }
  }
};
